# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

from cmath import exp
from socket import getnameinfo
from unittest import IsolatedAsyncioTestCase

import redis
import pytest
from preggy import expect
from thumbor.context import Context
from thumbor.config import Config


from tc_redis.storages.redis_storage import Storage as RedisStorage
from tests.fixtures.storage_fixtures import IMAGE_URL, IMAGE_BYTES, get_server


class RedisDBContext(IsolatedAsyncioTestCase):
    def setUp(self):
        self.sentinel = redis.Sentinel(
            [("localhost", 26380)],
            socket_timeout=1,
            sentinel_kwargs={"password": "superpassword"},
        )
        self.connection = self.sentinel.master_for(
            "masterinstance", socket_timeout=1, password="superpassword"
        )
        self.cfg = Config(
            REDIS_SENTINEL_STORAGE_INSTANCES="localhost:26380",
            REDIS_SENTINEL_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_STORAGE_MASTER_PASSWORD="superpassword",
            REDIS_SENTINEL_STORAGE_PASSWORD="superpassword",
            REDIS_STORAGE_MODE="sentinel",
        )
        self.storage = RedisStorage(
            Context(config=self.cfg, server=get_server("ACME-SEC"))
        )

    def test_should_be_instance_of_sentinel(self):
        expect(str(self.storage.get_storage())).to_equal(
            "Redis<SentinelConnectionPool<service=masterinstance(master)>"
        )


class CanStoreImage(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_be_in_catalog(self):
        await self.storage.put(IMAGE_URL % 1, IMAGE_BYTES)

        topic = self.connection.get(IMAGE_URL % 1)

        expect(topic).not_to_be_null()
        expect(topic).not_to_be_an_error()


class KnowsImageExists(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_exist(self):
        await self.storage.put(IMAGE_URL % 9999, IMAGE_BYTES)
        topic = await self.storage.exists(IMAGE_URL % 9999)
        expect(topic).not_to_be_null()
        expect(topic).not_to_be_an_error()
        expect(topic).to_be_true()


class KnowsImageDoesNotExist(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_not_exist(self):
        topic = await self.storage.exists(IMAGE_URL % 10000)
        expect(topic).not_to_be_null()
        expect(topic).not_to_be_an_error()
        expect(topic).to_be_false()


class CanRemoveImage(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_not_be_in_catalog(self):
        await self.storage.put(IMAGE_URL % 10001, IMAGE_BYTES)
        await self.storage.remove(IMAGE_URL % 10001)
        topic = self.connection.get(IMAGE_URL % 10001)
        expect(topic).not_to_be_an_error()
        expect(topic).to_be_null()


class CanReRemoveImage(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_not_be_in_catalog(self):
        await self.storage.remove(IMAGE_URL % 10001)
        topic = self.connection.get(IMAGE_URL % 10001)
        expect(topic).not_to_be_an_error()
        expect(topic).to_be_null()


class CanGetImage(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_not_be_null(self):
        await self.storage.put(IMAGE_URL % 2, IMAGE_BYTES)
        topic = await self.storage.get(IMAGE_URL % 2)
        expect(topic).not_to_be_null()
        expect(topic).not_to_be_an_error()

    @pytest.mark.asyncio
    async def test_should_have_proper_bytes(self):
        await self.storage.put(IMAGE_URL % 2, IMAGE_BYTES)
        topic = await self.storage.get(IMAGE_URL % 2)
        expect(topic).to_equal(IMAGE_BYTES)


class CanRaiseErrors(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_throw_an_exception(self):
        config = Config(
            REDIS_SENTINEL_STORAGE_INSTANCES="localhost:379",
            REDIS_SENTINEL_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_STORAGE_MASTER_PASSWORD="superpassword",
            REDIS_STORAGE_MODE="sentinel",
            REDIS_STORAGE_IGNORE_ERRORS=False,
        )
        storage = RedisStorage(
            context=Context(config=config, server=get_server("ACME-SEC")),
            shared_client=False,
        )

        try:
            topic = await storage.exists(IMAGE_URL % 2)
        except Exception as redis_error:
            expect(redis_error).not_to_be_null()
            expect(redis_error).to_be_an_error_like(redis.RedisError)


class IgnoreErrors(RedisDBContext):
    def setUp(self):
        super().setUp()

        self.cfg = Config(
            REDIS_SENTINEL_STORAGE_INSTANCES="localhost:379",
            REDIS_SENTINEL_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_STORAGE_MASTER_PASSWORD="superpassword",
            REDIS_STORAGE_MODE="sentinel",
            REDIS_STORAGE_IGNORE_ERRORS=True,
        )

        self.storage = RedisStorage(
            context=Context(config=self.cfg, server=get_server("ACME-SEC")),
            shared_client=False,
        )

    @pytest.mark.asyncio
    async def test_should_return_false(self):
        result = await self.storage.exists(IMAGE_URL % 2)
        expect(result).to_equal(False)
        expect(result).not_to_be_an_error()

    @pytest.mark.asyncio
    async def test_should_return_none(self):
        result = await self.storage.get(IMAGE_URL % 2)
        expect(result).to_equal(None)
        expect(result).not_to_be_an_error()


class RaisesIfInvalidConfig(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_be_an_error(self):
        config = Config(
            REDIS_SENTINEL_STORAGE_INSTANCES="localhost:26380",
            REDIS_SENTINEL_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_STORAGE_MASTER_PASSWORD="superpassword",
            REDIS_STORAGE_MODE="sentinel",
            STORES_CRYPTO_KEY_FOR_EACH_IMAGE=True,
        )
        storage = RedisStorage(Context(config=config, server=get_server("")))
        await storage.put(IMAGE_URL % 3, IMAGE_BYTES)

        try:
            await storage.put_crypto(IMAGE_URL % 3)
        except Exception as error:
            expect(error).to_be_an_error_like(RuntimeError)
            expect(error).to_have_an_error_message_of(
                "STORES_CRYPTO_KEY_FOR_EACH_IMAGE can't be True if no "
                "SECURITY_KEY specified"
            )


class GettingCryptoForANewImageReturnsNone(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_be_null(self):
        config = Config(
            REDIS_SENTINEL_STORAGE_INSTANCES="localhost:26380",
            REDIS_SENTINEL_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_STORAGE_MASTER_PASSWORD="superpassword",
            REDIS_STORAGE_MODE="sentinel",
            STORES_CRYPTO_KEY_FOR_EACH_IMAGE=True,
        )
        storage = RedisStorage(Context(config=config, server=get_server("ACME-SEC")))

        topic = await storage.get_crypto(IMAGE_URL % 9999)
        expect(topic).to_be_null()


class DoesNotStoreIfConfigSaysNotTo(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_be_null(self):
        await self.storage.put(IMAGE_URL % 5, IMAGE_BYTES)
        await self.storage.put_crypto(IMAGE_URL % 5)
        topic = await self.storage.get_crypto(IMAGE_URL % 5)
        expect(topic).to_be_null()


class CanStoreCrypto(RedisDBContext):
    def setUp(self):
        super().setUp()
        self.cfg = Config(
            REDIS_SENTINEL_STORAGE_INSTANCES="localhost:26380",
            REDIS_SENTINEL_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_STORAGE_MASTER_PASSWORD="superpassword",
            REDIS_STORAGE_MODE="sentinel",
            STORES_CRYPTO_KEY_FOR_EACH_IMAGE=True,
        )
        self.storage = RedisStorage(
            Context(config=self.cfg, server=get_server("ACME-SEC"))
        )

    @pytest.mark.asyncio
    async def test_should_not_be_null(self):
        await self.storage.put(IMAGE_URL % 6, IMAGE_BYTES)
        await self.storage.put_crypto(IMAGE_URL % 6)
        topic = await self.storage.get_crypto(IMAGE_URL % 6)
        expect(topic).not_to_be_null()
        expect(topic).not_to_be_an_error()

    @pytest.mark.asyncio
    async def test_should_have_proper_key(self):
        await self.storage.put(IMAGE_URL % 6, IMAGE_BYTES)
        await self.storage.put_crypto(IMAGE_URL % 6)
        topic = await self.storage.get_crypto(IMAGE_URL % 6)
        expect(topic).to_equal("ACME-SEC")


class CanStoreDetectorData(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_not_be_null(self):
        await self.storage.put(IMAGE_URL % 7, IMAGE_BYTES)
        await self.storage.put_detector_data(IMAGE_URL % 7, "some-data")
        topic = await self.storage.get_detector_data(IMAGE_URL % 7)
        expect(topic).not_to_be_null()
        expect(topic).not_to_be_an_error()

    @pytest.mark.asyncio
    async def test_should_equal_some_data(self):
        await self.storage.put(IMAGE_URL % 7, IMAGE_BYTES)
        await self.storage.put_detector_data(IMAGE_URL % 7, "some-data")
        topic = await self.storage.get_detector_data(IMAGE_URL % 7)
        expect(topic).to_equal("some-data")


class ReturnsNoneIfNoDetectorData(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_not_be_null(self):
        topic = await self.storage.get_detector_data(IMAGE_URL % 10000)
        expect(topic).to_be_null()


class ConnectToRedisWithoutPassword(RedisDBContext):
    def setUp(self):
        super().setUp()

        self.sentinel = redis.Sentinel([("localhost", 26379)], socket_timeout=1)
        self.connection = self.sentinel.master_for("masterinstance", socket_timeout=1)

        self.cfg = Config(
            REDIS_SENTINEL_STORAGE_INSTANCES="localhost:26379",
            REDIS_SENTINEL_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_STORAGE_MODE="sentinel",
        )

        self.storage = RedisStorage(
            context=Context(config=self.cfg, server=get_server("ACME-SEC")),
            shared_client=False,
        )

    @pytest.mark.asyncio
    async def test_should_be_in_catalog(self):
        await self.storage.put(IMAGE_URL % 1, IMAGE_BYTES)

        topic = self.connection.get(IMAGE_URL % 1)

        expect(topic).not_to_be_null()
        expect(topic).not_to_be_an_error()


class RedisModeInvalid(RedisDBContext):
    def setUp(self):
        super().setUp()

        self.cfg = Config(
            REDIS_SENTINEL_STORAGE_SERVER_HOST="localhost",
            REDIS_SENTINEL_STORAGE_SERVER_PORT=6379,
            REDIS_SENTINEL_STORAGE_SERVER_DB=0,
            REDIS_STORAGE_MODE="test",
        )
        self.ctx = Context(
            config=self.cfg,
            server=get_server("ACME-SEC"),
        )

    @pytest.mark.asyncio
    async def test_should_raises_attribute_error(self):
        with self.assertRaises(AttributeError) as error:
            RedisStorage(self.ctx)

        expect(str(error.exception)).to_equal(
            "Unknow value for REDIS_STORAGE_MODE test. See README for more information."
        )



# -*- coding: utf-8 -*-

import json
from datetime import datetime, timedelta

from redis import RedisError
from thumbor.storages import BaseStorage
from thumbor.utils import logger

from tc_redis.utils import on_exception
from tc_redis.base_storage import RedisBaseStorage


class Storage(BaseStorage, RedisBaseStorage):
    def __init__(self, context, shared_client=True):
        """Initialize the RedisStorage

        :param thumbor.context.Context shared_client: Current context
        :param boolean shared_client: When set to True a singleton client will
                                      be used.
        """

        BaseStorage.__init__(self, context)
        RedisBaseStorage.__init__(self, context, "storage")
        self.shared_client = shared_client
        self.storage = self.get_storage()

    def on_redis_error(self, fname, exc_type, exc_value):
        """Callback executed when there is a redis error.

        :param string fname: Function name that was being called.
        :param type exc_type: Exception type
        :param Exception exc_value: The current exception
        :returns: Default value or raise the current exception
        """

        self.storage = None
        self.set_shared_storage(None)

        if self.context.config.REDIS_STORAGE_IGNORE_ERRORS is True:
            logger.error(f"[REDIS_STORAGE] {exc_value}")
            if fname == "_exists":
                return False
            return None
        else:
            raise exc_value

    def __key_for(self, url):
        return f"thumbor-crypto-{url}"

    def __detector_key_for(self, url):
        return f"thumbor-detector-{url}"

    @on_exception(on_redis_error, RedisError)
    async def put(self, path, image_bytes):
        storage = self.get_storage()
        storage.set(path, image_bytes)
        storage.expireat(
            path,
            datetime.now()
            + timedelta(seconds=self.context.config.STORAGE_EXPIRATION_SECONDS),
        )

    @on_exception(on_redis_error, RedisError)
    async def put_crypto(self, path):
        if not self.context.config.STORES_CRYPTO_KEY_FOR_EACH_IMAGE:
            return

        if not self.context.server.security_key:
            raise RuntimeError(
                "STORES_CRYPTO_KEY_FOR_EACH_IMAGE can't be True if no "
                "SECURITY_KEY specified"
            )

        key = self.__key_for(path)
        self.get_storage().set(key, self.context.server.security_key)

    @on_exception(on_redis_error, RedisError)
    async def put_detector_data(self, path, data):
        key = self.__detector_key_for(path)
        self.get_storage().set(key, json.dumps(data))

    async def get_crypto(self, path):
        return self._get_crypto(path)

    @on_exception(on_redis_error, RedisError)
    def _get_crypto(self, path):
        if not self.context.config.STORES_CRYPTO_KEY_FOR_EACH_IMAGE:
            return None

        crypto = self.get_storage().get(self.__key_for(path))

        if not crypto:
            return None
        return crypto

    async def get_detector_data(self, path):
        return self._get_detector_data(path)

    @on_exception(on_redis_error, RedisError)
    def _get_detector_data(self, path):
        data = self.get_storage().get(self.__detector_key_for(path))

        if not data:
            return None
        return json.loads(data)

    async def exists(self, path):
        return self._exists(path)

    @on_exception(on_redis_error, RedisError)
    def _exists(self, path):
        return self.get_storage().exists(path)

    @on_exception(on_redis_error, RedisError)
    async def remove(self, path):
        self.get_storage().delete(path)

    async def get(self, path):
        @on_exception(self.on_redis_error, RedisError)
        def wrap():
            return self.get_storage().get(path)

        return wrap()



import os
from setuptools import setup, find_packages

try:
    import pypandoc

    long_description = pypandoc.convert("README.md", "rst")
except (IOError, ImportError):
    long_description = "Thumbor redis storage adapters"


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


TESTS_REQUIREMENTS = [
    "black==22.*,>=22.1.0",
    "preggy==1.*,>=1.4.4",
    "pyssim==0.*,>=0.4",
    "pytest==7.*,>=7.0.0",
    "pytest-cov==3.*,>=3.0.0",
    "pytest-asyncio==0.*,>=0.18.0",
]

RUNTIME_REQUIREMENTS = [
    "redis==4.*,>=4.1.3",
    "thumbor==7.*,>=7.0.6",
    "pre-commit==2.*,>=2.17.0",
]

setup(
    name="tc_redis",
    version="2.2.0",
    author="Thumbor Community",
    description=("Thumbor redis storage adapters"),
    license="MIT",
    keywords="thumbor redis",
    url="https://github.com/thumbor-community/redis",
    packages=find_packages(),
    long_description=long_description,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    install_requires=RUNTIME_REQUIREMENTS,
    extras_require={
        "all": RUNTIME_REQUIREMENTS,
        "tests": RUNTIME_REQUIREMENTS + TESTS_REQUIREMENTS,
    },
)



# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/globocom/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 globo.com timehome@corp.globo.com

from unittest import IsolatedAsyncioTestCase
from datetime import datetime, timedelta

import redis
import pytest
from preggy import expect
from thumbor.context import Context, RequestParameters
from thumbor.config import Config


from tc_redis.result_storages.redis_result_storage import Storage as RedisStorage
from tests.fixtures.storage_fixtures import IMAGE_URL, IMAGE_BYTES, get_server


class RedisDBContext(IsolatedAsyncioTestCase):
    def setUp(self):
        self.sentinel = redis.Sentinel(
            [("localhost", 26380)],
            socket_timeout=1,
            sentinel_kwargs={"password": "superpassword"},
        )
        self.connection = self.sentinel.master_for(
            "masterinstance", socket_timeout=1, password="superpassword"
        )
        self.cfg = Config(
            REDIS_SENTINEL_RESULT_STORAGE_INSTANCES="localhost:26380",
            REDIS_SENTINEL_RESULT_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_RESULT_STORAGE_MASTER_PASSWORD="superpassword",
            REDIS_SENTINEL_RESULT_STORAGE_PASSWORD="superpassword",
            REDIS_RESULT_STORAGE_MODE="sentinel",
            RESULT_STORAGE_EXPIRATION_SECONDS=60000,
        )
        self.ctx = Context(
            config=self.cfg,
            server=get_server("ACME-SEC"),
        )
        self.ctx.request = RequestParameters(
            url=IMAGE_URL % 2,
        )
        self.storage = RedisStorage(self.ctx)

    def test_should_be_instance_of_single_node(self):
        expect(str(self.storage.get_storage())).to_equal(
            "Redis<SentinelConnectionPool<service=masterinstance(master)>"
        )


class CanStoreImage(RedisDBContext):
    def setUp(self):
        super().setUp()

    @pytest.mark.asyncio
    async def test_should_be_in_catalog(self):
        await self.storage.put(IMAGE_BYTES)

        topic = self.connection.get(f"result:{IMAGE_URL % 2}")

        expect(topic).not_to_be_null()
        expect(topic).to_equal(IMAGE_BYTES)


class KnowsImageDoesNotExist(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_not_exist(self):
        self.ctx.request.url = IMAGE_URL % 10000
        topic = await self.storage.get()
        expect(topic).to_be_null()
        expect(topic).not_to_be_an_error()


class GetMaxAgeFromRedisEnv(RedisDBContext):
    def test_should_get_max_age_from_redis(self):
        self.ctx.request.max_age = 10
        topic = self.storage.get_max_age()
        expect(topic).not_to_be_null()
        expect(topic).to_equal(self.cfg.RESULT_STORAGE_EXPIRATION_SECONDS)


class GetMaxAgeFromRequest(RedisDBContext):
    def test_should_get_max_age_from_request(self):
        max_age = 0
        self.ctx.request.max_age = max_age
        topic = self.storage.get_max_age()
        expect(topic).not_to_be_null()
        expect(topic).to_equal(max_age)


class GetKeyFromRequest(RedisDBContext):
    def test_should_get_key_from_request(self):
        topic = self.storage.get_key_from_request()
        expect(topic).not_to_be_null()
        expect(topic).to_equal(f"result:{self.ctx.request.url}")


class GetWebpKeyFromRequest(RedisDBContext):
    def setUp(self):
        super().setUp()

        self.ctx.config.AUTO_WEBP = True
        self.ctx.request.accepts_webp = True

    def test_should_get_webp_from_request(self):
        topic = self.storage.get_key_from_request()
        expect(topic).not_to_be_null()
        expect(topic).to_equal(f"result:{self.ctx.request.url}/webp")


class CanGetAutoWebp(RedisDBContext):
    def setUp(self):
        super().setUp()

        self.ctx.config.AUTO_WEBP = True
        self.ctx.request.accepts_webp = True

    def test_should_get_auto_webp(self):
        topic = self.storage.get_key_from_request()
        expect(topic).not_to_be_null()
        expect(topic).to_equal(f"result:{self.ctx.request.url}/webp")


class CanNotGetAutoWebp(RedisDBContext):
    def setUp(self):
        super().setUp()

        self.ctx.config.AUTO_WEBP = True
        self.ctx.request.accepts_webp = False

    def test_should_not_get_auto_webp(self):
        topic = self.storage.get_key_from_request()
        expect(topic).not_to_be_null()
        expect(topic).to_equal(f"result:{self.ctx.request.url}")


class CanReadLastUpdatedFromStorage(RedisDBContext):
    def test_should_get_last_updated_from_storage(self):
        self.ctx.request.max_age = 0
        topic = self.storage.last_updated()
        expect(topic).not_to_be_null()
        expect(topic).to_equal(datetime.fromtimestamp(self.storage.start_time))


class CanReadLastUpdatedFromImage(RedisDBContext):
    def test_should_get_last_updated_from_image(self):
        self.connection.set("test_last_updated", IMAGE_BYTES)
        self.connection.expireat(
            "test_last_updated", datetime.now() + timedelta(seconds=1000)
        )
        self.ctx.request.max_age = 10

        topic = self.storage.last_updated()
        expect(topic).not_to_be_null()
        expect(topic).to_be_lesser_than(datetime.now())


class CanReadUnexpiryLastUpdated(RedisDBContext):
    def test_should_get_unexpire_last_updated(self):
        self.connection.set("test_last_updated", IMAGE_BYTES)
        self.ctx.request.max_age = 10
        ttl = self.connection.ttl("test_last_updated")
        topic = self.storage.last_updated()
        expect(topic).not_to_be_null()
        expect(ttl).to_equal(-1)


class CanRaiseErrors(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_throw_an_exception(self):
        config = Config(
            REDIS_SENTINEL_RESULT_STORAGE_INSTANCES="localhost:300",
            REDIS_SENTINEL_RESULT_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_RESULT_STORAGE_MASTER_PASSWORD="superpassword",
            REDIS_SENTINEL_RESULT_STORAGE_PASSWORD="superpassword",
            REDIS_RESULT_STORAGE_MODE="sentinel",
            REDIS_RESULT_STORAGE_IGNORE_ERRORS=False,
        )
        ctx = Context(
            config=config,
            server=get_server("ACME-SEC"),
        )
        ctx.request = RequestParameters(
            url=IMAGE_URL,
        )
        storage = RedisStorage(
            context=ctx,
            shared_client=False,
        )

        try:
            topic = await storage.get()
        except Exception as redis_error:
            expect(redis_error).not_to_be_null()
            expect(redis_error).to_be_an_error_like(redis.RedisError)


class CanIgnoreErrors(RedisDBContext):
    @pytest.mark.asyncio
    async def test_should_not_throw_an_exception(self):
        cfg = Config(
            REDIS_SENTINEL_RESULT_STORAGE_INSTANCES="localhost:300",
            REDIS_SENTINEL_RESULT_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_RESULT_STORAGE_MASTER_PASSWORD="superpassword",
            REDIS_SENTINEL_RESULT_STORAGE_PASSWORD="superpassword",
            REDIS_RESULT_STORAGE_MODE="sentinel",
            REDIS_RESULT_STORAGE_IGNORE_ERRORS=True,
        )
        ctx = Context(
            config=cfg,
            server=get_server("ACME-SEC"),
        )
        ctx.request = RequestParameters(
            url=IMAGE_URL % 10,
        )
        storage = RedisStorage(
            context=ctx,
            shared_client=False,
        )

        topic = await storage.get()
        expect(topic).to_equal(None)
        expect(topic).not_to_be_an_error()


class ConnectToRedisWithoutPassword(RedisDBContext):
    def setUp(self):
        super().setUp()

        self.sentinel = redis.Sentinel([("localhost", 26379)], socket_timeout=1)
        self.connection = self.sentinel.master_for("masterinstance", socket_timeout=1)

        self.cfg = Config(
            REDIS_SENTINEL_RESULT_STORAGE_INSTANCES="localhost:26379",
            REDIS_SENTINEL_RESULT_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_RESULT_STORAGE_MASTER_PASSWORD="superpassword",
            REDIS_RESULT_STORAGE_MODE="sentinel",
        )

        self.ctx = Context(
            config=self.cfg,
            server=get_server("ACME-SEC"),
        )
        self.ctx.request = RequestParameters(
            url=IMAGE_URL % 2,
        )
        self.storage = RedisStorage(self.ctx)

    @pytest.mark.asyncio
    async def test_should_be_in_catalog(self):
        await self.storage.put(IMAGE_BYTES)

        topic = self.connection.get(f"result:{IMAGE_URL % 2}")

        expect(topic).not_to_be_null()
        expect(topic).not_to_be_an_error()


class RedisModeInvalid(RedisDBContext):
    def setUp(self):
        super().setUp()

        self.cfg = Config(
            REDIS_SENTINEL_RESULT_STORAGE_SERVER_HOST="localhost",
            REDIS_SENTINEL_RESULT_STORAGE_SERVER_PORT=6379,
            REDIS_SENTINEL_RESULT_STORAGE_SERVER_DB=0,
            REDIS_RESULT_STORAGE_MODE="test",
        )
        self.ctx = Context(
            config=self.cfg,
            server=get_server("ACME-SEC"),
        )

    @pytest.mark.asyncio
    async def test_should_raises_attribute_error(self):
        with self.assertRaises(AttributeError) as error:
            RedisStorage(self.ctx)

        expect(str(error.exception)).to_equal(
            "Unknow value for REDIS_RESULT_STORAGE_MODE test. See README for more information."
        )


class ConnectToSentinelDatabase(RedisDBContext):
    def setUp(self):
        super().setUp()

        db1 = 1
        db2 = 2
        self.sentinel = redis.Sentinel([("localhost", 26379)], socket_timeout=1)
        self.connection_db1 = self.sentinel.master_for(
            "masterinstance", socket_timeout=1, db=db1
        )
        self.connection_db2 = self.sentinel.master_for(
            "masterinstance", socket_timeout=1, db=db2
        )

        self.storage_db1 = self.create_storage(db1)
        self.storage_db2 = self.create_storage(db2)

    def create_storage(self, db):
        cfg = Config(
            REDIS_SENTINEL_RESULT_STORAGE_INSTANCES="localhost:26379",
            REDIS_SENTINEL_RESULT_STORAGE_MASTER_INSTANCE="masterinstance",
            REDIS_SENTINEL_RESULT_STORAGE_MASTER_DB=db,
            REDIS_RESULT_STORAGE_MODE="sentinel",
        )

        ctx = Context(
            config=cfg,
            server=get_server("ACME-SEC"),
        )
        ctx.request = RequestParameters(
            url=IMAGE_URL % 2,
        )

        return RedisStorage(ctx, shared_client=False)

    @pytest.mark.asyncio
    async def test_should_be_in_catalog(self):
        await self.storage_db1.put(IMAGE_BYTES)

        topic = self.connection_db1.get(f"result:{IMAGE_URL % 2}")

        expect(topic).not_to_be_null()
        expect(topic).not_to_be_an_error()

    @pytest.mark.asyncio
    async def test_should_not_be_in_catalog(self):
        await self.storage_db1.put(IMAGE_BYTES)

        topic = self.connection_db2.get(f"result:{IMAGE_URL % 2}")

        expect(topic).to_be_null()
